from urllib import parse

import pandas

from dataHubPy.connector.database import DatabaseConnector


class MysqlConnector(DatabaseConnector):

    def __init__(
            self,
            host: str = None,
            port: int = None,
            username: str = None,
            password: str = None,
            database_name: str = None,
            if_write_table_exists: str = "fail",
            need_index: bool = False
    ) -> None:
        super().__init__(host, port, username, password, "", database_name,
                         if_write_table_exists,
                         need_index)

    def _connection_string(self):
        return "mysql+pymysql://{}:{}@{}:{}/{}".format(
            self.get_username(),
            parse.quote_plus(self.get_password()),
            self.get_host(),
            self.get_port(),
            self.get_database_name(),
        )

    def read_as_data_frame(self, table_name: str = "") -> pandas.DataFrame:
        with self._connect() as connection:
            return pandas.read_sql("select * from {}".format(table_name), connection)



import pandas

from minio import Minio
from io import BytesIO
from dataHubPy.connector.connector import Connector


class S3Connector(Connector):

    def __init__(
            self,
            endpoint: str = "",
            access_key: str = None,
            access_secret: str = None,
            bucket: str = None
    ) -> None:
        super().__init__()
        self.endpoint = endpoint
        self.access_key = access_key
        self.access_secret = access_secret
        self.bucket = bucket

    def get_endpoint(self):
        return self.endpoint

    def get_access_key(self):
        return self.access_key

    def get_access_secret(self):
        return self.access_secret

    def get_bucket(self):
        return self.bucket

    def init(self, info: dict):
        self.__dict__.update(info)
        return self

    def __eq__(self, other):
        if not isinstance(other, S3Connector):
            return False
        return (self.get_endpoint() == other.get_endpoint()
                and self.get_access_key() == other.get_access_key()
                and self.get_access_secret() == other.get_access_secret()
                and self.get_bucket() == other.get_bucket()
                )

    def read_as_data_frame(self, table_name: str = "") -> pandas.DataFrame:
        if not table_name or table_name == "":
            raise Exception("file name is None")
        client = self._connect()
        if client.bucket_exists(self.get_bucket()):
            file_object = client.get_object(self.get_bucket(), table_name)
            return pandas.read_csv(file_object)
        else:
            raise Exception("bucket is not exist.")

    def write_data_frame(self, df: pandas.DataFrame, table_name: str = "") -> None:
        if not table_name or table_name == "":
            raise Exception("you need to assign a output data item name")
        client = self._connect()
        if client.bucket_exists(self.get_bucket()):
            temp_csv = df.to_csv(index=False).encode("utf-8")
            client.put_object(
                self.get_bucket(),
                table_name,
                BytesIO(temp_csv),
                length=len(temp_csv),
                content_type="application/csv"
            )
        else:
            raise Exception("bucket is not exist.")

    def _connect(self):
        return Minio(
            endpoint=self.get_endpoint(),
            access_key=self.get_access_key(),
            secret_key=self.get_access_secret(),
            secure=False
        )



from dataHubPy.algorithm.algorithm import Algorithm


class AlgorithmFactory:
    def __init__(self, configuration_map: dict) -> None:
        super().__init__()
        self._configuration_map = configuration_map

    def algorithm(self) -> Algorithm:
        if self._configuration_map is None or self._configuration_map.get("type", "") == "":
            raise Exception("algorithm is None")
        return self._extract_algorithm()

    def _extract_algorithm(self):
        all_available_algos = {clazz.__name__: clazz for clazz in Algorithm.__subclasses__()}
        print("available algorithms : {}".format(all_available_algos))
        if self._configuration_map['type'] in all_available_algos.keys():
            selected_algo = all_available_algos[self._configuration_map['type']]
            return selected_algo(self._configuration_map)
        else:
            raise Exception("algorithm type({}) not support".format(self._configuration_map['type']))




import pandas

import sqlalchemy

from dataHubPy.connector.connector import Connector


class DatabaseConnector(Connector):

    def __init__(
            self,
            host: str,
            port: int,
            username: str,
            password: str,
            schema_name: str,
            database_name: str,
            if_write_table_exists: str = "fail",
            need_index: bool = False
    ) -> None:
        super().__init__()
        self.host = host
        self.port = port
        self.username = username
        self.password = password
        self.schema_name = schema_name
        self.database_name = database_name
        self.if_write_table_exists = if_write_table_exists
        self.need_index = need_index

    def read_as_data_frame(self, table_name: str = "") -> pandas.DataFrame:
        with self._connect() as connection:
            return pandas.read_sql("select * from {}.{}".format(self.get_schema_name(), table_name), connection)

    def write_data_frame(self, data: pandas.DataFrame, table_name: str = "") -> None:
        if not table_name or table_name == "":
            raise Exception("you need to assign a output data item name")
        with self._connect() as connection:
            data.to_sql(name=table_name, con=connection,
                        if_exists=self.get_if_write_table_exists(),
                        chunksize=10000,
                        index=self.get_need_index()
                        )

    def _connect(self):
        sql_engine = sqlalchemy.create_engine(self._connection_string())
        return sql_engine.connect()

    def _connection_string(self):
        raise Exception("not implement")

    def get_host(self):
        return self.host

    def get_port(self):
        return self.port

    def get_username(self):
        return self.username

    def get_password(self):
        return self.password

    def get_schema_name(self):
        return self.schema_name

    def get_database_name(self):
        return self.database_name

    def get_if_write_table_exists(self):
        return self.if_write_table_exists

    def get_need_index(self):
        return self.need_index

    def __eq__(self, other):
        if not isinstance(other, DatabaseConnector):
            return False
        return (self.get_host() == other.get_host()
                and self.get_port() == other.get_port()
                and self.get_username() == other.get_username()
                and self.get_password() == other.get_password()
                and self.get_schema_name() == other.get_schema_name()
                and self.get_database_name() == other.get_database_name()
                and self.get_if_write_table_exists() == other.get_if_write_table_exists()
                and self.get_need_index() == other.get_need_index()
                )



import base64
import tempfile

from flask import Flask, request

from dataHubPy.algorithm_executor import AlgorithmExecutor

flask_app = Flask(__name__)


@flask_app.route("/algorithms/invoke", methods=['POST'])
def invoke():
    request_json_data = request.json
    base64_yaml = request_json_data["base64_yaml"]
    with tempfile.NamedTemporaryFile(delete=False) as temp_yaml_file:
        temp_yaml_file.write(base64.b64decode(base64_yaml))
        temp_yaml_file.flush()
        algorithm_executor = AlgorithmExecutor(temp_yaml_file.name)
        result = algorithm_executor.execute()
        return result

@flask_app.route("/health/liveness", methods=['GET'])
def liveness():
    return "ok"


@flask_app.route("/health/readiness", methods=['GET'])
def readiness():
    return "ok"



import json
from typing import List

from dataHubPy.algorithm.factory import AlgorithmFactory
from dataHubPy.configuration.yaml_configuration import YamlConfiguration
from dataHubPy.connector.connector import Connector
from dataHubPy.connector.factory import ConnectorFactory
from dataHubPy.utils.format_util import NpEncoder


class AlgorithmExecutor:
    def __init__(self, config_path: str) -> None:
        super().__init__()
        configuration_map = YamlConfiguration(config_path).as_map()
        connector_factory = ConnectorFactory(configuration_map["connector"])
        self._input_connector_list = connector_factory.input_connector_list()
        self._output_connector_list = connector_factory.output_connector_list()
        algorithm_factory = AlgorithmFactory(configuration_map["algorithm"])
        self._algorithm = algorithm_factory.algorithm()

    def input_connector_list(self) -> List[Connector]:
        return self._input_connector_list

    def output_connector_list(self) -> List[Connector]:
        return self._output_connector_list

    def execute(self) -> str:
        result = self._algorithm.run(self._input_connector_list, self._output_connector_list, [])
        return json.dumps(result, ensure_ascii=False, cls=NpEncoder)



from typing import List

from dataHubPy.connector.connector import Connector


class Algorithm:
    _registry = {}

    def __init__(self, configuration: dict) -> None:
        super().__init__()
        self._input_data_item = configuration.get("input", {}).get("data", "")
        self._output_data_item = configuration.get("output", {}).get("data", "")

    def __init_subclass__(cls, **kwargs):
        super().__init_subclass__(**kwargs)
        cls._registry[cls.__name__] = cls

    def run(
            self,
            connector_in_list: List[Connector],
            connector_out_list: List[Connector],
            connector_cache_list: List[Connector]
    ) -> dict:
        raise Exception("not implement")

    def list_registry(self):
        return self._registry



import json
import numpy as np

TIMESTAMP_10 = r"^\d{10}(\.\d+)?$"
TIMESTAMP_13 = r"^\d{13}$"


class NpEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, np.integer):
            return int(obj)
        if isinstance(obj, np.floating):
            return float(obj)
        if isinstance(obj, np.ndarray):
            return obj.tolist()
        return json.JSONEncoder.default(self, obj)



class IterableData:
    def __init__(self) -> None:
        super().__init__()
        self._data = None

    def __iter__(self):
        return self

    def __next__(self):
        return self._data
